@@ -26,21 +26,43 @@ public class IconView : Gtk.Layout {
 	private bool scroll;
 	private double scroll_value;
 
+	/* suppress it sometimes */
+	bool suppress_scroll = false;
+	
+	// Zooming factor.
+	protected const double ZOOM_FACTOR = 1.2;
+
 	/* Width of the thumbnails. */
 	protected int thumbnail_width = 128;
+	protected const int MAX_THUMBNAIL_WIDTH = 256;
+	protected const int MIN_THUMBNAIL_WIDTH = 64;
 	public int ThumbnailWidth {
 		get {
 			return thumbnail_width;
 		}
-
 		set {
+			value = Math.Min(value, MAX_THUMBNAIL_WIDTH);
+			value = Math.Max(value, MIN_THUMBNAIL_WIDTH);
+
 			if (thumbnail_width != value) {
 				thumbnail_width = value;
 				QueueResize ();
+
+				if (ZoomChanged != null)
+					ZoomChanged (this, System.EventArgs.Empty);
 			}
 		}
 	}
 
+	public double Zoom {
+		get {
+			return ((double)(ThumbnailWidth - MIN_THUMBNAIL_WIDTH) / (double)(MAX_THUMBNAIL_WIDTH - MIN_THUMBNAIL_WIDTH));
+		}
+		set {
+			ThumbnailWidth = (int) ((value) * (MAX_THUMBNAIL_WIDTH - MIN_THUMBNAIL_WIDTH)) + MIN_THUMBNAIL_WIDTH;
+		}
+	}
+
 	protected double thumbnail_ratio = 4.0 / 3.0;
 	public double ThumbnailRatio {
 		get {
@@ -148,6 +170,9 @@ public class IconView : Gtk.Layout {
 	// Public events.
 	public delegate void DoubleClickedHandler (IconView view, int clicked_item);
 	public event DoubleClickedHandler DoubleClicked;
+	
+	public delegate void ZoomChangedHandler (object sender, System.EventArgs args);
+	public event ZoomChangedHandler ZoomChanged;
 
 	// Public API.
 	public IconView (IntPtr raw) : base (raw) {}
@@ -162,6 +187,7 @@ public class IconView : Gtk.Layout {
 		ButtonPressEvent += new ButtonPressEventHandler (HandleButtonPressEvent);
 		ButtonReleaseEvent += new ButtonReleaseEventHandler (HandleButtonReleaseEvent);
 		KeyPressEvent += new KeyPressEventHandler (HandleKeyPressEvent);
+		ScrollEvent += new ScrollEventHandler(HandleScrollEvent);
 
 		Destroyed += HandleDestroyed;
 
@@ -180,7 +206,7 @@ public class IconView : Gtk.Layout {
 		this.selection = new SelectionCollection (collection);
 		
 		collection.Changed += HandleChanged;
-		collection.ItemChanged += HandleItemChanged;
+		collection.ItemsChanged += HandleItemsChanged;
 
 		selection.DetailedChanged += HandleSelectionChanged;
 	}
@@ -198,13 +224,16 @@ public class IconView : Gtk.Layout {
 	{
 		// FIXME we should probably try to merge the selection forward
 		// but it needs some thought to be efficient.
+		suppress_scroll = true;
 		QueueResize ();
 	}
 	
-	private void HandleItemChanged (FSpot.IBrowsableCollection sender, int item)
+	private void HandleItemsChanged (FSpot.IBrowsableCollection sender, BrowsableArgs args)
 	{
-		UpdateThumbnail (item);
-		InvalidateCell (item);
+		foreach (int item in args.Items) {
+			UpdateThumbnail (item);
+			InvalidateCell (item);
+		}
 	}
 
 	//
@@ -240,7 +269,7 @@ public class IconView : Gtk.Layout {
 			this.selected_cells = new Hashtable ();
 			this.parent = collection;
 			this.parent.Changed += HandleParentChanged;
-			this.parent.ItemChanged += HandleParentItemChanged;
+			this.parent.ItemsChanged += HandleParentItemsChanged;
 		}
 
 		private void HandleParentChanged (IBrowsableCollection collection)
@@ -269,12 +298,26 @@ public class IconView : Gtk.Layout {
 
 		}
 
-		private void HandleParentItemChanged (IBrowsableCollection collection, int parent_index)
-		{
-			int local_index = this.IndexOf (parent_index);
-			//System.Console.WriteLine ("parent = {0} local = {1}", parent_index, local_index);
-			if (local_index >= 0 && this.ItemChanged != null)
-				this.ItemChanged (collection, local_index);
+		private void HandleParentItemsChanged (IBrowsableCollection collection, BrowsableArgs args)
+		{			
+			if (this.ItemsChanged == null)
+				return;
+
+			ArrayList local_ids = new ArrayList ();
+			foreach (int parent_index in args.Items) {
+				// If the item isn't part of the selection ignore it
+				if (!this.Contains (collection [parent_index]))
+					return;
+
+				int local_index = this.IndexOf (parent_index);
+				if (local_index >= 0)
+					local_ids.Add (local_index);
+			}
+
+			if (local_ids.Count == 0)
+				return;
+
+			ItemsChanged (this, new BrowsableArgs ((int [])local_ids.ToArray (typeof (int))));
 		}
 
 		public int [] Ids {
@@ -389,6 +432,26 @@ public class IconView : Gtk.Layout {
 			
 			SignalChange (ids);
 		}
+		
+		// Remove a range, except the start entry
+		public void Remove (int start, int end)
+		{
+			if (start == -1 || end == -1)
+				return;
+			
+			int current = Math.Min (start + 1, end);
+			int final = Math.Max (start - 1, end);				
+			int count = final - current + 1;
+			int [] ids = new int [count];
+
+			for (int i = 0; i < count; i++) {
+				this.Remove (current, false);
+				ids [i] = current;
+				current++;
+			}
+
+			SignalChange (ids);
+		}
 
 		public int IndexOf (int parent_index)
 		{
@@ -405,22 +468,37 @@ public class IconView : Gtk.Layout {
 		}
 
 		public void Remove (int cell)
+		{
+			Remove (cell, true);
+		}
+
+		private void Remove (int cell, bool notify)
 		{
 			IBrowsableItem item = parent [cell];
 			if (item != null)
-				this.Remove (item);
+				this.Remove (item, notify);
 
 		}
-		
+
 		public void Remove (IBrowsableItem item)
 		{
+			Remove (item, true);
+		}
+
+		private void Remove (IBrowsableItem item, bool notify)
+		{
+			if (item == null)
+				return;
+
 			int parent_index = (int) selected_cells [item];
 			selected_cells.Remove (item);
-			SignalChange (new int [] {parent_index});
+
+			if (notify)
+				SignalChange (new int [] {parent_index});
 		}
 
 		public event IBrowsableCollectionChangedHandler Changed;
-		public event IBrowsableCollectionItemChangedHandler ItemChanged;
+		public event IBrowsableCollectionItemsChangedHandler ItemsChanged;
 
 		public delegate void DetailedCollectionChanged (IBrowsableCollection collection, int [] ids);
 		public event DetailedCollectionChanged DetailedChanged;
@@ -444,15 +522,6 @@ public class IconView : Gtk.Layout {
 				DetailedChanged (this, ids);
 		}
 	}
-	
-	public int CurrentIdx {
-		get {
-			if (selection.Count == 1 && selection.Contains (FocusCell))
-				return FocusCell;
-			else 
-				return -1;
-		}
-	}
 
 	// Updating.
 	public void UpdateThumbnail (int thumbnail_num)
@@ -463,9 +532,7 @@ public class IconView : Gtk.Layout {
 		InvalidateCell (thumbnail_num);
 	}
 
-
 	// Cell Geometry
-
 	public int CellAtPosition (int x, int y)
 	{
 		return CellAtPosition (x, y, true);
@@ -498,7 +565,8 @@ public class IconView : Gtk.Layout {
 	
 	public int TopLeftVisibleCell ()
 	{
-		return CellAtPosition(BORDER_SIZE, (int)Vadjustment.Value + BORDER_SIZE + 8);
+		//return CellAtPosition(BORDER_SIZE, (int)Vadjustment.Value + BORDER_SIZE + 8);
+		return CellAtPosition(BORDER_SIZE, (int) (Vadjustment.Value + Allocation.Height * (Vadjustment.Value / Vadjustment.Upper)) + BORDER_SIZE + 8);
 	}
 
 	public void GetCellCenter (int cell_num, out int x, out int y)
@@ -627,15 +695,6 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
-	static Gdk.Rectangle Expand (Gdk.Rectangle src, int width)
-	{
-		src.X -= width;
-		src.Y -= width;
-		src.Width += width * 2;
-		src.Height += width * 2;
-		return src;
-	}
-
 	int ThrobExpansion (int cell, bool selected)
 	{
 		int expansion = 0;
@@ -681,7 +740,8 @@ public class IconView : Gtk.Layout {
 				    bounds.X, bounds.Y,
 				    bounds.Width - 1, bounds.Height - 1);
 		
-		Gdk.Rectangle focus = Expand (bounds, -3);
+		Gdk.Rectangle focus = Gdk.Rectangle.Inflate (bounds, -3, -3);
+
 		if (HasFocus && thumbnail_num == FocusCell) {
 			Style.PaintFocus(Style, BinWindow, 
 					 cell_state, area, 
@@ -691,14 +751,14 @@ public class IconView : Gtk.Layout {
 		}
 
 		Gdk.Rectangle region = Gdk.Rectangle.Zero;
-		Gdk.Rectangle image_bounds = Expand (bounds, - cell_border_width);
+		Gdk.Rectangle image_bounds = Gdk.Rectangle.Inflate (bounds, -cell_border_width, -cell_border_width);
 		int expansion = ThrobExpansion (thumbnail_num, selected);
 
 		Gdk.Pixbuf thumbnail = null;
 		if (entry != null)
 			thumbnail = entry.ShallowCopyPixbuf ();
 
-		if (Expand (image_bounds, expansion + 1).Intersect (area, out image_bounds) && thumbnail != null) {
+		if (Gdk.Rectangle.Inflate (image_bounds, expansion + 1, expansion + 1).Intersect (area, out image_bounds) && thumbnail != null) {
 			
 			PixbufUtils.Fit (thumbnail, ThumbnailWidth, ThumbnailHeight, 
 					 true, out region.Width, out region.Height);
@@ -709,7 +769,7 @@ public class IconView : Gtk.Layout {
 			if (region.Width != thumbnail.Width && region.Height != thumbnail.Height)
 				cache.Reload (entry, thumbnail_num, thumbnail.Width, thumbnail.Height);
 
-			region = Expand (region, expansion);
+			region = Gdk.Rectangle.Inflate (region, expansion, expansion);
 			Pixbuf temp_thumbnail;			
 			region.Width = System.Math.Max (1, region.Width);
 			region.Height = System.Math.Max (1, region.Height); 
@@ -747,7 +807,7 @@ public class IconView : Gtk.Layout {
 			region.Width = temp_thumbnail.Width;
 			region.Height = temp_thumbnail.Height;
 			
-			Gdk.Rectangle draw = Expand (region, 1);
+			Gdk.Rectangle draw = Gdk.Rectangle.Inflate (region, 1, 1);
 			
 			if (!temp_thumbnail.HasAlpha) 
 				Style.PaintShadow (Style, BinWindow, cell_state,
@@ -766,14 +826,15 @@ public class IconView : Gtk.Layout {
 								 draw.X, draw.Y);
 			}
 			
-			if (temp_thumbnail != thumbnail)
+			if (temp_thumbnail != thumbnail) {
 				temp_thumbnail.Dispose ();
+			}
 			
 		}
 		
-		if (thumbnail != null) 
+		if (thumbnail != null) {
 			thumbnail.Dispose ();
-		
+		}
 		Gdk.Rectangle layout_bounds = Gdk.Rectangle.Zero;
 		if (DisplayDates) {
 			string date;
@@ -847,8 +908,9 @@ public class IconView : Gtk.Layout {
 								      region.X, region.Y, 
 								      region.Width, region.Height,
 								      RgbDither.None, region.X, region.Y);
-					if (scaled_icon != icon)
+					if (scaled_icon != icon) {
 						scaled_icon.Dispose ();
+					}
 				}
 				tag_bounds.X += tag_bounds.Width + tag_icon_vspacing;
 			}
@@ -968,10 +1030,7 @@ public class IconView : Gtk.Layout {
 		offscreen.Subtract (onscreen);
 
 		PreloadRegion (offscreen, ystep);
-		offscreen.Destroy ();
-		//PreloadRegion (onscreen, ystep);
 		Preload (area, false);
-		onscreen.Destroy ();
 
 		y_offset = (int) Vadjustment.Value;
 		x_offset = (int) Hadjustment.Value;
@@ -1069,8 +1128,6 @@ public class IconView : Gtk.Layout {
 
 	private bool HandleThrobTimer () 
 	{
-		//Console.WriteLine ("throb out {1} {0}", throb_cell, 1 - Math.Cos (throb_state));
-
 		InvalidateCell (throb_cell);
 		if (throb_state++ < throb_state_max) {
 			return true;
@@ -1081,7 +1138,12 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
-	public void ScrollTo (int cell_num)
+ 	public void ScrollTo (int cell_num)
+ 	{
+		ScrollTo (cell_num, true);
+	}
+
+	public void ScrollTo (int cell_num, bool center)
 	{
 		if (!IsRealized)
 			return;
@@ -1095,12 +1157,24 @@ public class IconView : Gtk.Layout {
 		if (y + cell_height > adjustment.Upper)
 			UpdateLayout ();
 
-		adjustment.Value = y + cell_height / 2 - adjustment.PageSize / 2;
+		if (center)
+			adjustment.Value = y + cell_height / 2 - adjustment.PageSize / 2;
+		else
+			adjustment.Value = y;
 		
 		adjustment.ChangeValue ();
 
 	}
-
+	
+	public void ZoomIn ()
+	{
+		ThumbnailWidth = (int) (ThumbnailWidth * ZOOM_FACTOR);
+	}
+	
+	public void ZoomOut ()
+	{
+		ThumbnailWidth = (int) (ThumbnailWidth / ZOOM_FACTOR);
+	}
 
 	// Event handlers.
 
@@ -1135,11 +1209,24 @@ public class IconView : Gtk.Layout {
 		// Pixbuf option iformation to verify the thumbnail validity later
 		int width, height;
 		PixbufUtils.Fit (result, ThumbnailWidth, ThumbnailHeight, false, out width, out height);
-		if (result.Width != width && result.Height != height) {
+		if (result.Width > width && result.Height > height) {
 			//  System.Console.WriteLine ("scaling");
 			Gdk.Pixbuf temp = PixbufUtils.ScaleDown (result, width, height);
 			result.Dispose ();
 			result = temp;
+		} else if (result.Width < ThumbnailWidth && result.Height < ThumbnailHeight) {
+			// FIXME this is a workaround to handle images whose actual size is smaller than 
+			// the thumbnail size, it needs to be fixed at a different level.
+			Gdk.Pixbuf temp = new Gdk.Pixbuf (Gdk.Colorspace.Rgb, true, 8, ThumbnailWidth, ThumbnailHeight);
+			temp.Fill (0x00000000);
+			result.CopyArea (0, 0, 
+					 result.Width, result.Height, 
+					 temp, 
+					 (temp.Width - result.Width)/ 2,
+					 temp.Height - result.Height);
+
+			result.Dispose ();
+			result = temp;
 		}
 
 		cache.Update (entry, result);
@@ -1156,8 +1243,7 @@ public class IconView : Gtk.Layout {
 	}
 
 	public void InvalidateCell (int order) 
-	{
-		if (dead) return;
+	{		
 		Rectangle cell_area = CellBounds (order);
 		// FIXME where are we computing the bounds incorrectly
 		cell_area.Width -= 1;
@@ -1177,10 +1263,27 @@ public class IconView : Gtk.Layout {
 			args.Vadjustment.ValueChanged += new EventHandler (HandleAdjustmentValueChanged);
 	}
 
+	private void HandleScrollEvent(object sender, ScrollEventArgs args) 
+	{
+		// Activated only by Control + ScrollWheelUp/ScrollWheelDown
+		if (ModifierType.ControlMask != (args.Event.State & ModifierType.ControlMask))
+			return;
+		
+		if (args.Event.Direction == ScrollDirection.Up) {
+			ZoomIn ();
+			// stop event from propagating.
+			args.RetVal = true;			
+		} else if (args.Event.Direction == ScrollDirection.Down ) {
+			ZoomOut ();
+			args.RetVal = true;
+		}
+	}
+
 	protected override void OnSizeAllocated (Gdk.Rectangle allocation)
 	{
 		scroll_value = (Vadjustment.Value)/ (Vadjustment.Upper);
-		scroll = true;
+		scroll = !suppress_scroll;
+		suppress_scroll = false;
 		UpdateLayout (allocation);
 		base.OnSizeAllocated (allocation);
 	}
@@ -1246,10 +1349,8 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
-	void ContextMenu (ButtonPressEventArgs args, int cell_num)
+	protected virtual void ContextMenu (ButtonPressEventArgs args, int cell_num)
 	{
-		PhotoPopup popup = new PhotoPopup ();
-		popup.Activate (args.Event);
 	}
 	
 	private void HandleButtonReleaseEvent (object sender, ButtonReleaseEventArgs args)
@@ -1294,10 +1395,16 @@ public class IconView : Gtk.Layout {
 			FocusCell += cells_per_row;
 			break;
 		case Gdk.Key.Left:
-			FocusCell--;
+			if (control && shift)
+				FocusCell -= FocusCell % cells_per_row;
+			else
+				FocusCell--;
 			break;
 		case Gdk.Key.Right:
-			FocusCell++;
+			if (control && shift)
+				FocusCell += cells_per_row - (FocusCell % cells_per_row) - 1;
+			else
+				FocusCell++;
 			break;
 		case Gdk.Key.Up:
 			FocusCell -= cells_per_row;
@@ -1323,13 +1430,19 @@ public class IconView : Gtk.Layout {
 			return;		
 		}
 		
-		if (FocusCell < 0 || FocusCell > collection.Count - 1) {
-			FocusCell = focus_old;
+		FocusCell = Math.Max (FocusCell, 0);
+		FocusCell = Math.Min (FocusCell, collection.Count - 1);
+		
+		if (FocusCell == focus_old) {
 			args.RetVal = false;
-		}	
+			return;
+		}
 
 		if (shift) {
-			selection.Add (focus_old, FocusCell);
+			if (focus_old != FocusCell && selection.Contains (focus_old) && selection.Contains (FocusCell))
+				selection.Remove (FocusCell, focus_old);
+			else
+				selection.Add (focus_old, FocusCell);
 		} else if (!control) {
 			selection.Clear ();
 			selection.Add (FocusCell);
@@ -1338,7 +1451,6 @@ public class IconView : Gtk.Layout {
 		ScrollTo (FocusCell);
 	}
 
-	bool dead = false;
 	private void HandleDestroyed (object sender, System.EventArgs args)
 	{
 		cache.OnPixbufLoaded -= HandlePixbufLoaded;
