@@ -6,8 +6,96 @@ using GtkSharp;
 using System.Collections;
 using System.IO;
 using System;
+using Mono.Posix;
 
 public class ImportCommand : FSpot.GladeDialog {
+	private class PhotoGrid : Table {
+		const int NUM_COLUMNS = 5;
+		const int NUM_ROWS = 4;
+		
+	        const int CELL_WIDTH = 128;
+		const int CELL_HEIGHT = 96;
+
+		const int PADDING = 3;
+
+		Gtk.Image [] image_widgets;
+
+		int position = -1;
+		
+		/*
+		 * This controls whether or wraps as new photos are added.
+		 */
+		public bool Scroll = false;
+
+		public PhotoGrid () : base (NUM_ROWS, NUM_COLUMNS, true)
+		{
+			image_widgets = new Gtk.Image [NUM_ROWS * NUM_COLUMNS];
+
+			int i = 0;
+			for (uint j = 0; j < NUM_ROWS; j++) {
+				for (uint k = 0; k < NUM_COLUMNS; k ++) {
+					Gtk.Image image_widget = new Gtk.Image ();
+
+					image_widget = new Gtk.Image ();
+					image_widget.SetSizeRequest (CELL_WIDTH, CELL_HEIGHT);
+
+					Attach (image_widget, k, k + 1, j, j + 1, 0, 0, PADDING, PADDING);
+
+					image_widgets [i] = image_widget;
+
+					i ++;
+				}
+			}
+		}
+
+		private void ScrollIfNeeded ()
+		{
+			if (position < NUM_COLUMNS * NUM_ROWS)
+				return;
+
+			for (int i = 0; i < NUM_COLUMNS * (NUM_ROWS - 1); i ++) {
+				image_widgets [i].Pixbuf = image_widgets [i + NUM_COLUMNS].Pixbuf;
+			}
+
+			for (int i = NUM_COLUMNS * (NUM_ROWS - 1); i < NUM_COLUMNS * NUM_ROWS; i ++) {
+				// FIXME: Lame, apparently I can't set the Pixbuf property to null.
+				// GTK# bug?
+				image_widgets [i].Pixbuf = new Pixbuf (Colorspace.Rgb, false, 8, 1, 1);
+			}
+
+			position -= NUM_COLUMNS;
+		}
+
+		public void AddThumbnail (Pixbuf thumbnail)
+		{
+			Pixbuf scaled_thumbnail;
+
+			if (thumbnail.Width <= CELL_WIDTH && thumbnail.Height <= CELL_HEIGHT) {
+				scaled_thumbnail = thumbnail;
+			} else {
+				int thumbnail_width, thumbnail_height;
+
+				PixbufUtils.Fit (thumbnail, CELL_WIDTH, CELL_HEIGHT, false,
+						 out thumbnail_width, out thumbnail_height);
+				scaled_thumbnail = thumbnail.ScaleSimple (thumbnail_width, thumbnail_height,
+									  Gdk.InterpType.Bilinear);
+			}
+
+			position = (position + 1) % (NUM_COLUMNS * NUM_ROWS);
+			if (Scroll)
+				ScrollIfNeeded ();
+
+			image_widgets [position].Pixbuf = scaled_thumbnail;
+			if (Scroll)
+				position ++;
+
+
+			if (scaled_thumbnail != thumbnail)
+				scaled_thumbnail.Dispose ();
+			
+		}
+	}
+
 	internal class SourceItem : ImageMenuItem {
 		public ImportSource Source;
 
@@ -81,7 +169,7 @@ public class ImportCommand : FSpot.GladeDialog {
 			if (this.IsiPodPhoto)
 				this.Icon = PixbufUtils.LoadThemeIcon ("gnome-dev-ipod", 32);
 
-                        if (this.Icon == null)			if (this.Icon == null && this.IsCamera)
+			if (this.Icon == null && this.IsCamera)
 				this.Icon = PixbufUtils.LoadThemeIcon ("gnome-dev-media-cf", 32);
 
 			try {
@@ -114,10 +202,6 @@ public class ImportCommand : FSpot.GladeDialog {
 		}
 	}
 
-	//internal classs FolderSource : ImportSource {
-	//	private string path;
-	//}
-
 	internal class DriveSource : ImportSource {
 		public Gnome.Vfs.Drive Drive;
 		
@@ -221,7 +305,7 @@ public class ImportCommand : FSpot.GladeDialog {
 			if (source_count == 0) {
 				ImportSource source = new BrowseSource (Mono.Posix.Catalog.GetString ("(No Cameras Detected)"),
 									"emblem-camera");
-				SourceItem item = new SourceItem (source);
+			SourceItem item = new SourceItem (source);
 				item.Sensitive = false;
 				this.Append (item);
 			}
@@ -272,94 +356,6 @@ public class ImportCommand : FSpot.GladeDialog {
 		}
 	}		
 	
-	private class PhotoGrid : Table {
-		const int NUM_COLUMNS = 5;
-		const int NUM_ROWS = 4;
-		
-	        const int CELL_WIDTH = 128;
-		const int CELL_HEIGHT = 96;
-
-		const int PADDING = 3;
-
-		Gtk.Image [] image_widgets;
-
-		int position = -1;
-		
-		/*
-		 * This controls whether or wraps as new photos are added.
-		 */
-		public bool Scroll = false;
-
-		public PhotoGrid () : base (NUM_ROWS, NUM_COLUMNS, true)
-		{
-			image_widgets = new Gtk.Image [NUM_ROWS * NUM_COLUMNS];
-
-			int i = 0;
-			for (uint j = 0; j < NUM_ROWS; j++) {
-				for (uint k = 0; k < NUM_COLUMNS; k ++) {
-					Gtk.Image image_widget = new Gtk.Image ();
-
-					image_widget = new Gtk.Image ();
-					image_widget.SetSizeRequest (CELL_WIDTH, CELL_HEIGHT);
-
-					Attach (image_widget, k, k + 1, j, j + 1, 0, 0, PADDING, PADDING);
-
-					image_widgets [i] = image_widget;
-
-					i ++;
-				}
-			}
-		}
-
-		private void ScrollIfNeeded ()
-		{
-			if (position < NUM_COLUMNS * NUM_ROWS)
-				return;
-
-			for (int i = 0; i < NUM_COLUMNS * (NUM_ROWS - 1); i ++) {
-				image_widgets [i].Pixbuf = image_widgets [i + NUM_COLUMNS].Pixbuf;
-			}
-
-			for (int i = NUM_COLUMNS * (NUM_ROWS - 1); i < NUM_COLUMNS * NUM_ROWS; i ++) {
-				// FIXME: Lame, apparently I can't set the Pixbuf property to null.
-				// GTK# bug?
-				image_widgets [i].Pixbuf = new Pixbuf (Colorspace.Rgb, false, 8, 1, 1);
-			}
-
-			position -= NUM_COLUMNS;
-		}
-
-		public void AddThumbnail (Pixbuf thumbnail)
-		{
-			Pixbuf scaled_thumbnail;
-
-			if (thumbnail.Width <= CELL_WIDTH && thumbnail.Height <= CELL_HEIGHT) {
-				scaled_thumbnail = thumbnail;
-			} else {
-				int thumbnail_width, thumbnail_height;
-
-				PixbufUtils.Fit (thumbnail, CELL_WIDTH, CELL_HEIGHT, false,
-						 out thumbnail_width, out thumbnail_height);
-				scaled_thumbnail = thumbnail.ScaleSimple (thumbnail_width, thumbnail_height,
-									  Gdk.InterpType.Bilinear);
-			}
-
-			position = (position + 1) % (NUM_COLUMNS * NUM_ROWS);
-			if (Scroll)
-				ScrollIfNeeded ();
-
-			image_widgets [position].Pixbuf = scaled_thumbnail;
-			if (Scroll)
-				position ++;
-
-
-			if (scaled_thumbnail != thumbnail)
-				scaled_thumbnail.Dispose ();
-			
-		}
-	}
-
-
 	[Glade.Widget] Gtk.OptionMenu tag_option_menu;
 	[Glade.Widget] Gtk.OptionMenu source_option_menu;
 	[Glade.Widget] Gtk.ScrolledWindow icon_scrolled;
@@ -418,14 +414,6 @@ public class ImportCommand : FSpot.GladeDialog {
 		progress_bar.Fraction = (double) count / System.Math.Max (total, 1);
 	}
 
-	private void HandleCollectionChanged (FSpot.IBrowsableCollection col)
-	{
-		if (collection.Count == 1) {
-			photo_view.Item.Index = -1;
-			photo_view.Item.Index = 0;
-		}
-	}
-
 	private void HandleTraySelectionChanged (FSpot.IBrowsableCollection coll) 
 	{
 		if (tray.Selection.Count > 0)
@@ -474,7 +462,6 @@ public class ImportCommand : FSpot.GladeDialog {
 		}
 	}
 
-
 	private int DoImport (ImportBackend imp)
 	{
 		if (collection == null)
@@ -637,30 +624,21 @@ public class ImportCommand : FSpot.GladeDialog {
 		this.Dialog.WindowPosition = Gtk.WindowPosition.CenterOnParent;
 		this.Dialog.Response += HandleDialogResponse;
 
-		MenuItem attach_item = new MenuItem (Mono.Posix.Catalog.GetString ("Select Tag"));
-		TagMenu tagmenu = new TagMenu (null, MainWindow.Toplevel.Database.Tags);
+		CreateTagMenu ();
+		
+	        AllowFinish = false;
 		
 		this.Dialog.DefaultResponse = ResponseType.Ok;
 		
 		//import_folder_entry.Activated += HandleEntryActivate;
-
-		tagmenu.TagSelected += HandleTagMenuSelected;
-		tagmenu.ShowAll ();
-		tagmenu.Populate (true);
-		tagmenu.Prepend (attach_item);
-
-	        AllowFinish = false;
-		
 		recurse_check.Toggled += HandleRecurseToggled;
 		copy_check.Toggled += HandleRecurseToggled;
 
-		tag_option_menu.Menu = tagmenu;
 		SourceMenu menu = new SourceMenu ();
 		source_option_menu.Menu = menu;
 
 		collection = new FSpot.PhotoList (new Photo [0]);
-		collection.Changed += HandleCollectionChanged;
-		tray = new TrayView (collection);
+		tray = new FSpot.ScalingIconView (collection);
 		tray.Selection.Changed += HandleTraySelectionChanged;
 		icon_scrolled.SetSizeRequest (200, 480);
 		icon_scrolled.Add (tray);
@@ -742,6 +720,31 @@ public class ImportCommand : FSpot.GladeDialog {
 		}
 	}
 
+	private void CreateTagMenu ()
+	{
+		TagMenu tagmenu = new TagMenu (null, MainWindow.Toplevel.Database.Tags);
+
+		tagmenu.Append (new MenuItem (Catalog.GetString ("Select Tag")));
+		
+		tagmenu.Populate (true);
+		
+		GtkUtil.MakeMenuSeparator (tagmenu);
+		GtkUtil.MakeMenuItem (tagmenu, Catalog.GetString ("Create New Tag"),
+				"f-spot-new-tag", MainWindow.Toplevel.HandleCreateNewCategoryCommand, true);
+		
+		tagmenu.TagSelected += HandleTagMenuSelected;
+		MainWindow.Toplevel.Database.Tags.ItemsAdded += HandleTagsAdded;
+		
+		tagmenu.ShowAll ();
+		tag_option_menu.Menu = tagmenu;
+	}
+
+	private void HandleTagsAdded (object sender, DbItemEventArgs args)
+	{
+		// The user added a tag, so recreate the tag menu
+		CreateTagMenu ();
+	}
+
 	public void Cancel ()
 	{
 		if (importer != null) {
@@ -782,11 +785,11 @@ public class ImportCommand : FSpot.GladeDialog {
 		return ImportFromPaths (store, paths, null);
 	}
 
-    public int ImportFromPaths (PhotoStore store, string [] paths, bool copy)
-    {
-        return ImportFromPaths (store, paths, null, copy);
-    }
-
+	public int ImportFromPaths (PhotoStore store, string [] paths, bool copy)
+	{
+		return ImportFromPaths (store, paths, null, copy);
+	}
+	
 	public int ImportFromPaths (PhotoStore store, string [] paths, Tag [] tags)
 	{
 		return ImportFromPaths (store, paths, tags, false);
