@@ -138,7 +138,7 @@ namespace FSpot {
 			byte [] thumb_data = PixbufUtils.Save (thumbnail, "jpeg", null, null);
 
 			// now update the exif data
-			exif_data.Data = thumb_data;
+			ExifData.Data = thumb_data;
 			thumbnail.Dispose ();
 
 			Exif.ExifEntry e;
@@ -220,7 +220,8 @@ namespace FSpot {
 		public void SetDateTimeOriginal (DateTime time)
 		{
 			Exif.ExifEntry e = ExifData.LookupFirst (Exif.Tag.DateTimeOriginal);
-			e.SetData (time);
+			if (e != null)
+				e.SetData (time);
 		}
 
 		public override System.DateTime Date {
