@@ -1,5 +1,6 @@
 using Gtk;
 using System;
+using Mono.Posix;
 
 namespace FSpot {
 	public class SingleView {
@@ -171,6 +172,53 @@ namespace FSpot {
 			metadata_dialog.ShowAll ();
 			metadata_dialog.Destroyed += HandleMetadataDestroyed;
 		}
+
+		private void HandleAbout (object sender, System.EventArgs args)
+		{
+			MainWindow.HandleAbout (sender, args);
+		}
+
+		private void HandleNewWindow (object sender, System.EventArgs args)
+		{
+			/* FIXME this needs to register witth the core */
+			new SingleView (path);
+		}
+
+		private void HandleOpenFolder (object sender, System.EventArgs args)
+		{
+			FileChooserDialog chooser = new FileChooserDialog (Catalog.GetString ("Open Folder"),
+									   window,
+									   FileChooserAction.SelectFolder);
+
+			chooser.AddButton (Stock.Cancel, ResponseType.Cancel);
+			chooser.AddButton (Stock.Open, ResponseType.Ok);
+
+			chooser.SetFilename (collection.Path);
+
+			int response = chooser.Run ();
+
+			if ((ResponseType) response == ResponseType.Ok) 
+				collection.Path = chooser.Filename;
+
+			chooser.Destroy ();
+		}
+
+		private void HandleOpen (object sender, System.EventArgs args)
+		{
+			FileChooserDialog chooser = new FileChooserDialog (Catalog.GetString ("Open"),
+									   window,
+									   FileChooserAction.Open);
+
+			chooser.AddButton (Stock.Cancel, ResponseType.Cancel);
+			chooser.AddButton (Stock.Open, ResponseType.Ok);
+
+			chooser.SetFilename (collection.Path);
+			int response = chooser.Run ();
+			if ((ResponseType) response == ResponseType.Ok) 
+				collection.Path = chooser.Filename;
+
+			chooser.Destroy ();
+		}
 		
 		private void HandleMetadataDestroyed (object sender, System.EventArgs args)
 		{
@@ -247,7 +295,6 @@ namespace FSpot {
 			this.Window.Destroy ();
 		}
 
-
 		private void SavePreferences  ()
 		{
 			int width, height;
