@@ -120,7 +120,7 @@ public class TagCommands {
 			this.Dialog.DefaultResponse = ResponseType.Ok;
 
 			this.Dialog.Title = Mono.Posix.Catalog.GetString ("Create New Tag");
-			prompt_label.Text = Mono.Posix.Catalog.GetString ("Name of new tag:");
+			prompt_label.Text = Mono.Posix.Catalog.GetString ("Name of New Tag:");
 
 			PopulateCategoryOptionMenu ();
 			this.Category = default_category;
@@ -200,7 +200,7 @@ public class TagCommands {
 				ok_button.Sensitive = false;
 				already_in_use_label.Markup = "";
 			} else if (TagNameExistsInCategory (name, db.Tags.RootCategory)
-				   && name != orig_name) {
+				   && String.Compare(name, orig_name, true) != 0) {
 				ok_button.Sensitive = false;
 				already_in_use_label.Markup = "<small>" + Mono.Posix.Catalog.GetString ("This name is already in use") + "</small>";
 			} else {
@@ -251,7 +251,7 @@ public class TagCommands {
 				     parent = parent.Category)
 					label_builder.Append ("  ");
 				
-				label_builder.Append (category.Name);
+				label_builder.Append (category.Name.Replace ("_", "__"));
 				
 				// FIXME escape underscores.
 				MenuItem item = new MenuItem (label_builder.ToString ());
@@ -273,7 +273,7 @@ public class TagCommands {
 			this.Dialog.DefaultResponse = ResponseType.Ok;
 
 			this.Dialog.Title = Mono.Posix.Catalog.GetString ("Edit Tag");
-			prompt_label.Text = Mono.Posix.Catalog.GetString ("Tag name:");
+			prompt_label.Text = Mono.Posix.Catalog.GetString ("Tag Name:");
 
 			orig_name = last_valid_name = t.Name;
 			tag_name_entry.Text = t.Name;
