@@ -2,9 +2,10 @@ using Gtk;
 using Gnome;
 using System;
 using System.IO;
+using System.Runtime.InteropServices;
+using System.Text;
 
 public class Driver {
-
 	public static void Main (string [] args)
 	{
 		bool view_only = false;
@@ -13,18 +14,42 @@ public class Driver {
 		Program program = null;
 		FSpot.CoreControl control = null;
 
+		SetProcessName (FSpot.Defines.PACKAGE);
+		
+		foreach (string arg in args) {
+			if (arg == "--help") {
+				System.Console.WriteLine ("Usage f-spot [OPTION. ..]\n");
+				System.Console.WriteLine ("  --import [uri]\t\t\timport from the given uri");
+				System.Console.WriteLine ("  --view <file>\t\t\t\tview a file or directory ");
+				System.Console.WriteLine ("  --shutdown\t\t\t\tshutdown a running f-spot server");
+				System.Console.WriteLine ("  --help\t\t\t\tview this message");
+				System.Console.WriteLine ("");
+
+				program = new Program (FSpot.Defines.PACKAGE, 
+						       FSpot.Defines.VERSION, 
+						       Modules.UI, args);
+				return;
+			}
+		}
+
+		/* 
+		 * FIXME we need to inialize gobject before making the dbus calls, we'll go 
+		 * ahead and do it like this for now.
+		 */ 
+		program = new Program (FSpot.Defines.PACKAGE, 
+				       FSpot.Defines.VERSION, 
+				       Modules.UI, args);		
+		
 		try {
 			control = FSpot.Core.FindInstance ();
 			System.Console.WriteLine ("Found active FSpot server: {0}", control);
+			program = null;
 		} catch (System.Exception e) { 
 			System.Console.WriteLine ("Starting new FSpot server");
 		}
 
 		if (control == null) {
 			FSpot.Core core = null;
-			program = new Program (FSpot.Defines.PACKAGE, 
-					       FSpot.Defines.VERSION, 
-					       Modules.UI, args);
 			
 			Gnome.Vfs.Vfs.Initialize ();
 			StockIcons.Initialize ();
@@ -34,12 +59,7 @@ public class Driver {
 			
 			// FIXME: Error checking is non-existant here...
 			
-			string base_directory = FSpot.Global.BaseDirectory;
-			if (! File.Exists (base_directory))
-				Directory.CreateDirectory (base_directory);
-			
-			Db db = new Db (Path.Combine (base_directory, "photos.db"), true);
-			core = new FSpot.Core (db);
+			core = new FSpot.Core ();
 
 			try {
 				core.RegisterServer ();
@@ -47,9 +67,9 @@ public class Driver {
 				System.Console.WriteLine (e.ToString ());
 			}
 
-			empty = db.Empty;
+			empty = FSpot.Core.Database.Empty;
 			control = core;
-		}			
+		}
 			
 		for (int i = 0; i < args.Length; i++) {
 			switch (args [i]) {
@@ -80,4 +100,21 @@ public class Driver {
 		if (program != null)
 			program.Run ();
 	}
+
+	[DllImport("libc")]
+	private static extern int prctl(int option, string name, ulong arg3,
+					ulong arg4, ulong arg5);
+	
+	public static void SetProcessName(string name)
+	{
+		try {
+			if(prctl(15 /* PR_SET_NAME */, name,
+				 0, 0, 0) != 0) {
+				throw new ApplicationException("Error setting process name: " +
+							       Mono.Unix.Native.Stdlib.GetLastError());
+			}
+		} catch (DllNotFoundException de) {
+			/* noop */
+		}
+	}
 }
